/*
 * deltacloud.h: management of interface to deltacloud
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#ifndef __MC_CLOUD_DELTACLOUD_H__
#define __MC_CLOUD_DELTACLOUD_H__

#include "util/types.h"
#include "util/list.h"
#include "deployable.h"
#include "instance.h"

#include <sys/types.h>

// we use libdeltacloud to communicate w/ the server
#include <libdeltacloud/libdeltacloud.h>

#define MC_DELTACLOUD_URL_BUFLEN (256 + 1)

#define MC_DELTACLOUD_HOSTNAME_BUFLEN (250)

#define MC_DELTACLOUD_PORT_BUFLEN (250)

/**
 * dcConnection: 
 *
 * a dcConnection is a structure representing a connection
 * to a deltacloud instance
 */
typedef struct _dcConnection {
  char url[MC_DELTACLOUD_URL_BUFLEN];    /* url of deltacloud server to connect to */
  struct deltacloud_api deltacloud_connection;  /* connection to deltacloud server */
} dcConnection;

/**
 * dcConnectionPtr:
 *
 * a dcConnectionPtr is a pointer to the dcConnection structure
 */
typedef dcConnection *dcConnectionPtr;

/**
 * create hostname / port into url
 */
int dcCreateDefaultUrl(char* url);

/**
 * create hostname / port into url
 */
int dcCreateUrl(char* url, const char* hostname, const char* port);

/** 
 * establish a new dc connection
 */
int dcConnect(dcConnectionPtr connection, char* username, char* password);

/** 
 * disconnect dc connection
 */
int dcDisconnect(dcConnectionPtr connection);

#define MC_DELTACLOUD_DRIVER_BUFLEN (256 + 1)

/**
 * dcServer:
 *
 * a dcServer is a structure representing an embedded deltacloud server
 */
typedef struct _dcServer {
  pid_t pid;                                  /* process id of running server */
  char driver[MC_DELTACLOUD_DRIVER_BUFLEN];   /* driver the server is running */
} dcServer;

/**
 * dcServerPtr:
 *
 * a dcServerPtr is a pointer to the dcServer structure
 */
typedef dcServer *dcServerPtr;

/**
 * launch embedded deltacloud server
 */
int dcServerLaunch(dcServerPtr *server, char* driver);

/**
 * terminate embedded deltacloud server
 */
int dcServerTerm(dcServerPtr server);

/** return boolean indicating if the specified dcServer is running
 */
BOOL dcServerIsRunning(dcServerPtr server);

/**
 * retrieve first server corresponding to target out of list
 */
dcServerPtr dcServerListRetrieveDriver(mcLinkedListPtr servers, char* driver);

/**
 * retrieve deployable on provider corresponding to id
 */
int dcRetrieveProviderDeployable(mcDeployablePtr deployable, mcProviderPtr provider, const char* id);

/**
 * create and return new instance on provider using deltacloud
 */
int dcCreateInstance(mcInstancePtr instance, mcProviderPtr provider);

/**
 * stop instance on provider using deltacloud
 */
int dcTerminateInstance(mcInstancePtr instance);

#endif  // #define __MC_CLOUD_DELTACLOUD_H__
