/*
 * deployable.c: core driver for deployable apis
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include "util/types.h"
#include "util/path.h"
#include "deployable.h"
#include "provider.h"
#include "imagefactory.h"
#include "deltacloud.h"

#include <regex.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

mcDeployablePtr mcDeployableDefineFromFile(char *filePath){
  mcDeployablePtr deployable;
  char* dir, *name;

  deployable = malloc(sizeof(mcDeployable));
  split_path_file(&dir, &name, filePath);
  strncpy(deployable->name, name, MC_DEPLOYABLE_NAME_BUFLEN);
  strncpy(deployable->file_path, filePath, MC_DEPLOYABLE_FILE_PATH_BUFLEN);
  return deployable;
}

int mcDeployableUndefine(mcDeployablePtr deployable){
  free(deployable);
  return 0;
}

int deployableBuild(mcDeployablePtr deployable, mcProviderPtr provider){
  imgfacDeployableBuild(provider->dc_server->driver, deployable->file_path, deployable->id);
  return 0;
}

int deployablePush(mcDeployablePtr deployable, mcProviderPtr provider){
  BOOL is_pushed;
  mcDeployable retrieved_deployable;

  imgfacDeployablePush(provider->dc_server->driver, deployable->id);

  // wait until image appears on provider
  is_pushed = FALSE;
  do{
    dcRetrieveProviderDeployable(&retrieved_deployable, provider, deployable->id);
    is_pushed = (strncmp(retrieved_deployable.id, deployable->id, MC_DEPLOYABLE_ID_BUFLEN) != 0);
    if(!is_pushed)
      sleep(MC_DEPLOYABLE_PUSH_POLL_DELAY);
  }while(!is_pushed);
  return 0;
}
