/*
 * hardwareprofile.c: core driver for hardware profile apis
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include "hardwareprofile.h"

#include <string.h>
#include <stdlib.h>

int mcProviderNumHardwareProfiles(mcProviderPtr provider){
  return mcLinkedListNumElements(&provider->hardwareProfiles);
}

int mcProviderListHardwareProfiles(mcProviderPtr provider, char **names, int maxnames){
  int iter;
  mcLinkedListNodePtr current_profile;
  mcHardwareProfilePtr current_hwp;

  iter = 0;
  current_profile = provider->hardwareProfiles.first;
  while(iter < maxnames && current_profile != NULL){
    current_hwp = (mcHardwareProfilePtr)current_profile->value;
    names[iter] = current_hwp->name;
    iter += 1;
    current_profile = current_profile->next;
  }

  return 0;
}

mcHardwareProfilePtr mcProviderHardwareProfileAt(mcProviderPtr provider, const int index){
  return (mcHardwareProfilePtr)mcLinkedListElementAt(&provider->hardwareProfiles, index)->value;
}

mcHardwareProfilePtr mcProviderHardwareProfileLookupByName(mcProviderPtr provider, const char* name){
  mcLinkedListNodePtr current_node;
  mcHardwareProfilePtr current_profile;

  current_node = provider->hardwareProfiles.first;
  while(current_node != NULL){
    current_profile = (mcHardwareProfilePtr)current_node->value;
    if(strncmp(current_profile->name, name, MC_HARDWARE_PROFILE_NAME_BUFLEN) == 0)
      return current_profile;
    current_node = current_node->next;
  }
  return NULL;
}

int mcHardwareProfileGetInfo(mcHardwareProfilePtr hwp, mcHardwareProfileInfoPtr* info){
  *info = &hwp->info;
  return 0;
}


char* dcHardwareProfileGetNamedParam(struct deltacloud_hardware_profile* dc_hwp, const char* label){
  struct deltacloud_property* cp;

  cp = dc_hwp->properties;
  while(cp != NULL){
    if(strncmp(cp->name, label, MC_HARDWARE_PROFILE_NAME_BUFLEN) == 0)
      return cp->value;
    cp = cp->next;
  }
  return NULL;
}


int dcHardwareProfileToMcHardwareProfile(struct deltacloud_hardware_profile *dc_hwp, mcHardwareProfilePtr mc_hwp){
    char *param;
    strncpy(mc_hwp->name, dc_hwp->name, MC_HARDWARE_PROFILE_NAME_BUFLEN);

    param = dcHardwareProfileGetNamedParam(dc_hwp, "cpus");
    if(param != NULL)
      mc_hwp->info.cpus    = atoi(param);

    param = dcHardwareProfileGetNamedParam(dc_hwp, "memory");
    if(param != NULL)
      mc_hwp->info.memory  = atoi(param);

    param = dcHardwareProfileGetNamedParam(dc_hwp, "storage");
    if(param != NULL)
      mc_hwp->info.storage = atoi(param);

    param = dcHardwareProfileGetNamedParam(dc_hwp, "arch");
    if(param != NULL)
      strncpy(mc_hwp->info.arch, param, MC_HARDWARE_PROFILE_ARCH_BUFLEN);
    return 0;
}

int dcHardwareProfilesToMcHardwareProfiles(struct deltacloud_hardware_profile *dc_hwps, mcLinkedListPtr mc_hwps){
  struct deltacloud_hardware_profile *current_dc;
  mcHardwareProfilePtr current_mc;

  current_dc = dc_hwps;
  while(current_dc != NULL){
     current_mc = malloc(sizeof(mcHardwareProfile));
     dcHardwareProfileToMcHardwareProfile(current_dc, current_mc);
     mcLinkedListAppend(mc_hwps, current_mc);
     current_dc = current_dc->next;
  }
  return 0;
}

int deallocateMcHardwareProfiles(mcLinkedListPtr mc_hwps){
  mcLinkedListNodePtr current_node;

  current_node = mc_hwps->first;
  while(current_node != NULL){
    free((mcHardwareProfilePtr)current_node->value);
    current_node = current_node->next;
  }
  mcLinkedListRemoveAll(mc_hwps);
  return 0;
}
