/*
 * hardwareprofile.h: core driver for hardware profile apis
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

/**
 * MC_HARDWARE_PROFILE_NAME_BUFLEN:
 *
 * Macro providing the maximum length of the mcHardwareProfile name string.
 * 
 */

#ifndef __MC_CLOUD_HARDWARE_PROFILE_H__
#define __MC_CLOUD_HARDWARE_PROFILE_H__

#include "provider.h"

#include <libdeltacloud/libdeltacloud.h>

// we use libdeltacloud to communicate w/ the server
#include <libdeltacloud/libdeltacloud.h>

#define MC_HARDWARE_PROFILE_NAME_BUFLEN (256 + 1)

/**
 * _mcHardwareProfile: internal representation of hardware profile
 *
 */
struct _mcHardwareProfile{
  mcProviderPtr provider;      /* provider to which hardware profile belongs to */
  mcHardwareProfileInfo info;  /* hardware profile attributes */
  char name[MC_HARDWARE_PROFILE_NAME_BUFLEN];
};

/**
 *  helper to extract a single named param from a hwp
 */
char* dcHardwareProfileGetNamedParam(struct deltacloud_hardware_profile* dc_hwp, const char* label);

/**
 * convert at deltacloud hardware profile to a mc hardware profile
 */
int dcHardwareProfileToMcHardwareProfile(struct deltacloud_hardware_profile *dc_hwp, mcHardwareProfilePtr mc_hwp);

/**
 * convert a list deltacloud of hardware profiles to a linked list of new mc hwps
 */
int dcHardwareProfilesToMcHardwareProfiles(struct deltacloud_hardware_profile *dc_hwps, mcLinkedListPtr mc_hwps);

/**
 * deallocate contents of a list of mcHardwareProfiles
 */
int deallocateMcHardwareProfiles(mcLinkedListPtr mc_hwps);

#endif // #define __MC_CLOUD_HARDWARE_PROFILE_H__
