/*
 * imagefactory.c: management of interface to imagefactory
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <regex.h>
#include <stdio.h>
#include <sys/wait.h>

#include "imagefactory.h"
#include "deployable.h"
#include "mycloud.h"

static char const* IMAGEFACTORY_COMMAND      = "/usr/bin/imagefactory";
static char const* AEOLUS_IMAGE_COMMAND      = "/usr/bin/aeolus-image";

int imgfacServerLaunch(imgfacServerPtr server){
  pid_t pID;
  char* argv[0];
  int status;
  
  status = 0;
  pID = fork();
  if(pID == 0){       // child
    execv(IMAGEFACTORY_COMMAND, argv);

  }else if(pID < 0){  // failed to fork
    status = 1;

  }else{              // parent
    server->pid = pID;

  }

  return status;
}

int imgfacServerTerm(imgfacServerPtr server){
  //kill(server.pid, SIGHUP);
  //sleep(3);

  //kill(server.pid, SIGINT);
  //sleep(3);

  kill(server->pid, SIGQUIT);
  sleep(1);

  return 0;
}

BOOL imgfacServerIsRunning(imgfacServerPtr server){
  int process_status;
  pid_t spid;

  spid = waitpid(server->pid, &process_status, WNOHANG);
  return spid == 0; // -1 = error ; server->pid = not running
}

int imgfacDeployableBuild(const char* target, const char* template_file, char* uuid){
  char argv[30 + sizeof(AEOLUS_IMAGE_COMMAND) + MC_PROVIDER_TARGET_BUFLEN + MC_DEPLOYABLE_FILE_PATH_BUFLEN];
  FILE* stdout_pipe;
  char stdout_buffer[MC_DEPLOYABLE_BUILD_OUTPUT_BUFLEN];
  int read_bytes;

  regex_t *image_id_regex;
  size_t nmatch;
  regmatch_t pmatch[2];

  strcpy(uuid, "");

  // run build command in pipe
  sprintf(argv, "%s build --target %s --template %s", 
                AEOLUS_IMAGE_COMMAND, target, template_file);
  stdout_pipe = popen(argv, "r");
  sleep(1);
  pclose(stdout_pipe);

  // read output, parse for text
  read_bytes = fread(stdout_buffer, 1, MC_DEPLOYABLE_BUILD_OUTPUT_BUFLEN, stdout_pipe);
printf("!!! %i %s\n", read_bytes, argv);
  if(read_bytes > 0){
    nmatch = 2; image_id_regex = NULL;
    regcomp(image_id_regex, "^\\s*Image:\\s*([0-9a-zA-Z\\-]*).*", REG_EXTENDED | REG_NOSUB);
    if(!regexec(image_id_regex, stdout_buffer, nmatch, pmatch, 0)){
      strncpy(uuid, stdout_buffer + pmatch[1].rm_so, pmatch[1].rm_eo - pmatch[1].rm_so);
    }
    regfree(image_id_regex);
  }

  return 0;
}

int imgfacDeployablePush(char* provider, char* uuid){
  pid_t pid;
  char* argv[6];

  // fork/exec command to push deployable to provider
  pid = fork();
  if(pid == -1){       // error
  }else if(pid == 0){  // child
    argv[0] = "push";
    argv[1] = "--provider";
    argv[2] = provider;
    argv[3] = "--id";
    argv[4] = uuid;
    argv[5] = NULL;
    execv(AEOLUS_IMAGE_COMMAND, argv);
  }else{               // parent
    waitpid(pid, NULL, 0);
  }

  return 0;
}
