/*
 * provider.c: core driver for provider apis
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include "provider.h"

#include <stdlib.h>

int providerInit(mcProviderPtr provider){
  provider->is_connected = FALSE;

  // create new linked list for profiles
  mcLinkedListInit(&provider->hardwareProfiles);
  return 0;
}

int providerDeinit(mcProviderPtr provider){
  //mcLinkedListNodePtr  cnode;
  //mcHardwareProfilePtr chwp;

  //// deinitialize each hardware profile
  //cnode = provider->hardwareProfiles.first;
  //while(cnode != NULL){
  //  chwp = (mcHardwareProfilePtr)cnode->value;
  //  hardwareProfileDeinit(chwp);
  //  cnode = cnode->next;
  //}
  mcLinkedListDeinit(&provider->hardwareProfiles);
  return 0;
}

int mcProviderConnect(mcProviderConnectionPtr provider_connection){
  struct deltacloud_hardware_profile *hwps;
  mcLinkedListNodePtr current_node;
  mcHardwareProfilePtr current_mc_hwp;
  mcProviderPtr provider;

  provider_connection->provider = malloc(sizeof(mcProvider));
  provider = provider_connection->provider;

  // initialize the provider
  providerInit(provider);

  // startup embedded deltacloud server for provider
  dcServerLaunch(&provider->dc_server, provider_connection->type);

  // create a new connection to it
  provider->connection.deltacloud_connection.driver = provider_connection->type;
  dcCreateDefaultUrl(provider->connection.url);
  dcConnect(&provider->connection, provider_connection->username, provider_connection->password);

  provider->is_connected = TRUE;

  // retrieve the list of hardware profiles
  deltacloud_get_hardware_profiles(&provider->connection.deltacloud_connection, &hwps);
  dcHardwareProfilesToMcHardwareProfiles(hwps, &provider->hardwareProfiles);

  // set provider on them
  current_node = provider->hardwareProfiles.first;
  while(current_node != NULL){
     current_mc_hwp = (mcHardwareProfilePtr)current_node->value;
     current_mc_hwp->provider = provider;
     current_node = current_node->next;
  }
  return 0;
}

int mcProviderDisconnect(mcProviderConnectionPtr provider_connection){
  mcProviderPtr provider;
  
  provider = provider_connection->provider;

  provider->is_connected = FALSE;

  // TODO terminate running server?

  // deallocate providers
  deallocateMcHardwareProfiles(&provider->hardwareProfiles);

  // disconnect from deltacloud server
  dcDisconnect(&provider->connection);

  // teminate embedded server
  // this will be done in configDeinit so deferring here
  // dcServerTerm(&provider_connection->provider->dc_server);

  // deinit provider object
  providerDeinit(provider);
  free(provider);
  return 0;
}
