/*
 * list.c: linked list utility helper
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include "types.h"
#include "list.h"

#include <stdlib.h>

int mcLinkedListInit(mcLinkedListPtr list){
  list->first = NULL;
  list->last  = NULL;
  return 0;
}

int mcLinkedListDeinit(mcLinkedListPtr list){
  return mcLinkedListRemoveAll(list);
}

BOOL mcLinkedListHasValue(mcLinkedListPtr list, void* value){
   mcLinkedListNodePtr node;

   node = list->first;
   while(node != NULL){
     if(node->value == value)
       return TRUE;
     node = node->next;
   }
   return FALSE;
}

int mcLinkedListAppend(mcLinkedListPtr list, void* item){
   mcLinkedListNodePtr node;

   node = malloc(sizeof(mcLinkedListNode));
   node->value = item;
   node->prev  = list->last;
   node->next  = NULL;

   if(list->last != NULL)
     list->last->next = node;
   list->last = node;
   if(list->first == NULL)
     list->first = node;
   return 0;
}

int mcLinkedListRemove(mcLinkedListPtr list, void* item){
   mcLinkedListNodePtr current;

   current = list->first;
   while(current != NULL){
     if(current->value == item){
       if(current->prev != NULL)
         current->prev->next = current->next;
       else
         list->first = current->next;

       if(current->next != NULL)
         current->next->prev = current->prev;
       else
         list->last = current->prev;

       current->prev = NULL;
       current->next = NULL;
       free(current);

       return 0;
     }
     current = current->next;
   }

   return 1; // item not found
}

int mcLinkedListRemoveAll(mcLinkedListPtr list){
  mcLinkedListNodePtr current, next;

  current = list->first;
  while(current != NULL){
    next = current->next;
    mcLinkedListRemove(list, current->value);
    current = next;
  }

  return 0;
}

int mcLinkedListNumElements(mcLinkedListPtr list){
   int count;
   mcLinkedListNodePtr current;

   count = 0;
   current = list->first;
   while(current != NULL){
     ++count;
     current = current->next;
   }
   return count;
}

mcLinkedListNodePtr mcLinkedListElementAt(mcLinkedListPtr list, const int index){
  int num_elements, count;
  mcLinkedListNodePtr current_element;

  num_elements = mcLinkedListNumElements(list);

  if(index < 0 || index >= num_elements)
    return NULL;

  count = 0;
  current_element = list->first;
  while(count < index){
    ++count;
    current_element = current_element->next;
  }
  return current_element;
}
