/*
 * list.h: linked list utility helper
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#ifndef __MC_CLOUD_UTIL_LIST_H__
#define __MC_CLOUD_UTIL_LIST_H__

#include "types.h"

/**
 * mcLinkedListNode:
 *
 * represents a node on a linked list
 */
typedef struct _mcLinkedListNode{
  struct _mcLinkedListNode *prev;  /* previous node in the list */
  struct _mcLinkedListNode *next;  /* next node in the list */
  void* value;               /* value of the node */

} mcLinkedListNode;

/**
 * mcLinkedListNodePtr:
 *
 * represents a pointer to a mcLinkedListNode
 */
typedef mcLinkedListNode *mcLinkedListNodePtr;

/**
 * mcLinkedList:
 *
 * represents a collection of mcLinkedListNodes
 */
typedef struct _mcLinkedList{
  mcLinkedListNodePtr first;
  mcLinkedListNodePtr last;

} mcLinkedList;


/**
 * mcLinkedListPtr:
 *
 * represents a poitner to a mcLinkedList
 */
typedef mcLinkedList *mcLinkedListPtr;

/**
 * initialize the specified linked list
 */
int mcLinkedListInit(mcLinkedListPtr list);

/**
 * deinitialize the specified linked list,
 * it is up to the invoker to deinitialize each individual node
 */
int mcLinkedListDeinit(mcLinkedListPtr list);

/**
 * return true/false if the list has the specified value
 */
BOOL mcLinkedListHasValue(mcLinkedListPtr list, void* value);

/**
 * append item to list
 */
int mcLinkedListAppend(mcLinkedListPtr list, void* item);

/**
 * remove item from list
 */
int mcLinkedListRemove(mcLinkedListPtr list, void* item);

/**
 * removes all item from list
 */
int mcLinkedListRemoveAll(mcLinkedListPtr list);

/**
 * return the number of elements in a list
 */
int mcLinkedListNumElements(mcLinkedListPtr list);

/**
 * return the element at the specified position in the list
 */
mcLinkedListNodePtr mcLinkedListElementAt(mcLinkedListPtr list, const int index);

#endif
